from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class FluentCommentsApp(AppConfig):
    name = 'fluent_comments'
    verbose_name = _("Comments")

    def ready(self):
        # This installs the comment_will_be_posted signal
        import fluent_comments.receivers  # noqa



"""
API for :ref:`custom-comment-app-api`
"""
default_app_config = 'fluent_comments.apps.FluentCommentsApp'

form_class = None
model_class = None

# following PEP 440
__version__ = "2.1"


def get_model():
    """
    Return the model to use for commenting.
    """
    global model_class
    if model_class is None:
        from fluent_comments.models import FluentComment
        # Our proxy model that performs select_related('user') for the comments
        model_class = FluentComment

    return model_class


def get_form():
    """
    Return the form to use for commenting.
    """
    global form_class
    from fluent_comments import appsettings
    if form_class is None:
        if appsettings.FLUENT_COMMENTS_FORM_CLASS:
            from django.utils.module_loading import import_string
            form_class = import_string(appsettings.FLUENT_COMMENTS_FORM_CLASS)
        else:
            from fluent_comments.forms import FluentCommentForm
            form_class = FluentCommentForm

    return form_class



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

AKISMET_API_KEY = getattr(settings, 'AKISMET_API_KEY', None)
AKISMET_BLOG_URL = getattr(settings, 'AKISMET_BLOG_URL', None)  # Optional, to override auto detection
AKISMET_IS_TEST = getattr(settings, 'AKISMET_IS_TEST', False)   # Enable in case of testing

CRISPY_TEMPLATE_PACK = getattr(settings, 'CRISPY_TEMPLATE_PACK', 'bootstrap')

USE_THREADEDCOMMENTS = 'threadedcomments' in settings.INSTALLED_APPS

FLUENT_COMMENTS_REPLACE_ADMIN = getattr(settings, "FLUENT_COMMENTS_REPLACE_ADMIN", True)

FLUENT_CONTENTS_USE_AKISMET = getattr(settings, 'FLUENT_CONTENTS_USE_AKISMET', bool(AKISMET_API_KEY))  # enable when an API key is set.
FLUENT_COMMENTS_DEFAULT_MODERATOR = getattr(settings, 'FLUENT_COMMENTS_DEFAULT_MODERATOR', 'default')
FLUENT_COMMENTS_USE_EMAIL_NOTIFICATION = getattr(settings, 'FLUENT_COMMENTS_USE_EMAIL_NOTIFICATION', True)  # enable by default
FLUENT_COMMENTS_CLOSE_AFTER_DAYS = getattr(settings, 'FLUENT_COMMENTS_CLOSE_AFTER_DAYS', None)
FLUENT_COMMENTS_MODERATE_BAD_WORDS = getattr(settings, 'FLUENT_COMMENTS_MODERATE_BAD_WORDS', ())
FLUENT_COMMENTS_MODERATE_AFTER_DAYS = getattr(settings, 'FLUENT_COMMENTS_MODERATE_AFTER_DAYS', None)
FLUENT_COMMENTS_AKISMET_ACTION = getattr(settings, 'FLUENT_COMMENTS_AKISMET_ACTION', 'soft_delete')  # or 'moderate', 'delete, 'soft_delete'

FLUENT_COMMENTS_FIELD_ORDER = tuple(getattr(settings, 'FLUENT_COMMENTS_FIELD_ORDER', ()) or ())
FLUENT_COMMENTS_EXCLUDE_FIELDS = tuple(getattr(settings, 'FLUENT_COMMENTS_EXCLUDE_FIELDS', ()) or ())
FLUENT_COMMENTS_FORM_CLASS = getattr(settings, 'FLUENT_COMMENTS_FORM_CLASS', None)
FLUENT_COMMENTS_FORM_CSS_CLASS = getattr(settings, 'FLUENT_COMMENTS_FORM_CSS_CLASS', 'comments-form form-horizontal')
FLUENT_COMMENTS_LABEL_CSS_CLASS = getattr(settings, 'FLUENT_COMMENTS_LABEL_CSS_CLASS', 'col-sm-2')
FLUENT_COMMENTS_FIELD_CSS_CLASS = getattr(settings, 'FLUENT_COMMENTS_FIELD_CSS_CLASS', 'col-sm-10')

# Compact style settings
FLUENT_COMMENTS_COMPACT_FIELDS = getattr(settings, 'FLUENT_COMMENTS_COMPACT_FIELDS', ('name', 'email', 'url'))
FLUENT_COMMENTS_COMPACT_GRID_SIZE = getattr(settings, 'FLUENT_COMMENTS_COMPACT_GRID_SIZE', 12)
FLUENT_COMMENTS_COMPACT_COLUMN_CSS_CLASS = getattr(settings, 'FLUENT_COMMENTS_COMPACT_COLUMN_CSS_CLASS', "col-sm-{size}")


if FLUENT_COMMENTS_AKISMET_ACTION not in ('auto', 'moderate', 'soft_delete', 'delete'):
    raise ImproperlyConfigured("FLUENT_COMMENTS_AKISMET_ACTION can be 'auto', 'moderate', 'soft_delete' or 'delete'")

if FLUENT_COMMENTS_EXCLUDE_FIELDS or FLUENT_COMMENTS_FORM_CLASS or FLUENT_COMMENTS_FIELD_ORDER:
    # The exclude option only works when our form is used.
    # Allow derived packages to inherit our form class too.
    if not hasattr(settings, 'COMMENTS_APP') or settings.COMMENTS_APP == 'comments':
        raise ImproperlyConfigured("To use django-fluent-comments, also specify: COMMENTS_APP = 'fluent_comments'")



from collections import OrderedDict

import django_comments
from django.core.exceptions import ImproperlyConfigured
from fluent_comments import appsettings
from fluent_comments.forms.helper import CommentFormHelper
from fluent_comments.forms.helper import SubmitButton, PreviewButton  # noqa, import at old class location too

if appsettings.USE_THREADEDCOMMENTS:
    from threadedcomments.forms import ThreadedCommentForm as base_class
else:
    from django_comments.forms import CommentForm as base_class


class AbstractCommentForm(base_class):
    """
    The comment form, applies various settings.
    """

    #: Helper for {% crispy %} template tag
    helper = CommentFormHelper()

    def __init__(self, *args, **kwargs):
        self.is_preview = kwargs.pop('is_preview', False)
        super(AbstractCommentForm, self).__init__(*args, **kwargs)

        # Remove fields from the form.
        # This has to be done in the constructor, because the ThreadedCommentForm
        # inserts the title field in the __init__, instead of the static form definition.
        for name in appsettings.FLUENT_COMMENTS_EXCLUDE_FIELDS:
            try:
                self.fields.pop(name)
            except KeyError:
                raise ImproperlyConfigured(
                    "Field name '{0}' in FLUENT_COMMENTS_EXCLUDE_FIELDS is invalid, "
                    "it does not exist in the '{1}' class.".format(
                        name, self.__class__.__name__
                     )
                )

        if appsettings.FLUENT_COMMENTS_FIELD_ORDER:
            ordering = (
                 CommentFormHelper.BASE_FIELDS_TOP +
                 appsettings.FLUENT_COMMENTS_FIELD_ORDER +
                 CommentFormHelper.BASE_FIELDS_END
            )
            self._reorder_fields(ordering)

    def _reorder_fields(self, ordering):
        new_fields = OrderedDict()
        for name in ordering:
            new_fields[name] = self.fields[name]
        self.fields = new_fields

    def get_comment_model(self):
        # Provide the model used for comments. When this doesn't match
        # the sender used by django_comments.moderation.Moderator used in
        # `comment_will_be_posted.connect(..., sender=...)`, it will break moderation.
        #
        # Since ThreadedCommentForm overrides this method, it breaks moderation
        # with COMMENTS_APP="fluent_comments". Hence, by default let this match
        # the the model the app is configured with.
        return django_comments.get_model()

    def get_comment_create_data(self, *args, **kwargs):
        # Fake form data for excluded fields, so there are no KeyError exceptions
        for name in appsettings.FLUENT_COMMENTS_EXCLUDE_FIELDS:
            self.cleaned_data[name] = ""

        # Pass args, kwargs for django-contrib-comments 1.8, which accepts a ``site_id`` argument.
        return super(AbstractCommentForm, self).get_comment_create_data(*args, **kwargs)



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Button, Submit
from crispy_forms.utils import TEMPLATE_PACK
from django import forms
from django.forms.widgets import Input
from django.utils.translation import ugettext_lazy as _
from django_comments import get_form_target

from fluent_comments import appsettings


class CommentFormHelper(FormHelper):
    """
    The django-crispy-forms configuration that handles form appearance.
    The default is configured to show bootstrap forms nicely.
    """
    form_tag = False  # we need to define the form_tag
    form_id = 'comment-form-ID'
    form_class = 'js-comments-form {0}'.format(appsettings.FLUENT_COMMENTS_FORM_CSS_CLASS)
    label_class = appsettings.FLUENT_COMMENTS_LABEL_CSS_CLASS
    field_class = appsettings.FLUENT_COMMENTS_FIELD_CSS_CLASS
    render_unmentioned_fields = True  # like honeypot and security_hash

    BASE_FIELDS_TOP = ('content_type', 'object_pk', 'timestamp', 'security_hash')
    BASE_FIELDS_END = ('honeypot',)

    if appsettings.USE_THREADEDCOMMENTS:
        BASE_FIELDS_TOP += ('parent',)

    @property
    def form_action(self):
        return get_form_target()  # reads get_form_target from COMMENTS_APP

    def __init__(self, form=None):
        super(CommentFormHelper, self).__init__(form=form)
        if form is not None:
            # When using the helper like this, it could generate all fields.
            self.form_id = 'comment-form-{0}'.format(form.target_object.pk)
            self.attrs = {
                'data-object-id': form.target_object.pk,
            }


class CompactLabelsCommentFormHelper(CommentFormHelper):
    """
    Compact labels in the form, show them as placeholder text instead.

    .. note::
        Make sure that the :attr:`layout` attribute is defined and
        it has fields added to it, otherwise the placeholders don't appear.

    The text input can easily be resized using CSS like:

    .. code-block: css

        @media only screen and (min-width: 768px) {
          form.comments-form input.form-control {
            width: 50%;
          }
        }

    """
    form_class = CommentFormHelper.form_class.replace('form-horizontal', 'form-vertical') + ' comments-form-compact'
    label_class = 'sr-only'
    field_class = ''

    def render_layout(self, form, context, template_pack=TEMPLATE_PACK):
        """
        Copy any field label to the ``placeholder`` attribute.
        Note, this method is called when :attr:`layout` is defined.
        """
        # Writing the label values into the field placeholders.
        # This is done at rendering time, so the Form.__init__() could update any labels before.
        # Django 1.11 no longer lets EmailInput or URLInput inherit from TextInput,
        # so checking for `Input` instead while excluding `HiddenInput`.
        for field in form.fields.values():
            if field.label and \
                    isinstance(field.widget, (Input, forms.Textarea)) and \
                    not isinstance(field.widget, forms.HiddenInput):
                field.widget.attrs['placeholder'] = u"{0}:".format(field.label)

        return super(CompactLabelsCommentFormHelper, self).render_layout(form, context, template_pack=template_pack)


class SubmitButton(Submit):
    """
    The submit button to add to the layout.

    Note: the ``name=post`` is mandatory, it helps the
    """

    def __init__(self, text=_("Post Comment"), **kwargs):
        super(SubmitButton, self).__init__(name='post', value=text, **kwargs)


class PreviewButton(Button):
    """
    The preview button to add to the layout.

    Note: the ``name=post`` is mandatory, it helps the
    """
    input_type = 'submit'

    def __init__(self, text=_("Preview"), **kwargs):
        kwargs.setdefault('css_class', 'btn-default')
        super(PreviewButton, self).__init__(name='preview', value=text, **kwargs)



"""
The comment signals are handled to fallback to a default moderator.

This avoids not checking for spam or sending email notifications
for comments that bypassed the moderator registration
(e.g. posting a comment on a different page).

This is especially useful when a django-fluent-contents "CommentsAreaItem"
element is added to a random page subclass (which is likely not registered).
"""
import logging

import django_comments
from django.core.exceptions import ImproperlyConfigured
from django.dispatch import receiver
from django.utils.module_loading import import_string
from django_comments import signals

from fluent_comments import appsettings, moderation

logger = logging.getLogger(__name__)


def load_default_moderator():
    """
    Find a moderator object
    """
    if appsettings.FLUENT_COMMENTS_DEFAULT_MODERATOR == 'default':
        # Perform spam checks
        return moderation.FluentCommentsModerator(None)
    elif appsettings.FLUENT_COMMENTS_DEFAULT_MODERATOR == 'deny':
        # Deny all comments not from known registered models.
        return moderation.AlwaysDeny(None)
    elif str(appsettings.FLUENT_COMMENTS_DEFAULT_MODERATOR).lower() == 'none':
        # Disables default moderator
        return moderation.NullModerator(None)
    elif '.' in appsettings.FLUENT_COMMENTS_DEFAULT_MODERATOR:
        return import_string(appsettings.FLUENT_COMMENTS_DEFAULT_MODERATOR)(None)
    else:
        raise ImproperlyConfigured(
            "Bad FLUENT_COMMENTS_DEFAULT_MODERATOR value. Provide default/deny/none or a dotted path"
        )


default_moderator = load_default_moderator()
CommentModel = django_comments.get_model()


@receiver(signals.comment_will_be_posted)
def on_comment_will_be_posted(sender, comment, request, **kwargs):
    """
    Make sure both the Ajax and regular comments are checked for moderation.
    This signal is also used to link moderators to the comment posting.
    """
    content_object = comment.content_object
    moderator = moderation.get_model_moderator(content_object.__class__)
    if moderator and comment.__class__ is not CommentModel:
        # Help with some hard to diagnose problems. The default Django moderator connects
        # to the configured comment model. When this model differs from the signal sender,
        # the the form stores a different model then COMMENTS_APP provides.
        moderator = None
        logger.warning(
            "Comment of type '%s' was not moderated by '%s', "
            "because the parent '%s' has a moderator installed for '%s' instead",
            comment.__class__.__name__, moderator.__class__.__name__,
            content_object.__class__.__name__, CommentModel.__name__
        )

    if moderator is None:
        logger.info(
            "Using default moderator for comment '%s' on parent '%s'",
            comment.__class__.__name__, content_object.__class__.__name__
        )
        _run_default_moderator(comment, content_object, request)


def _run_default_moderator(comment, content_object, request):
    """
    Run the default moderator
    """
    # The default moderator will likely not check things like "auto close".
    # It can still provide akismet and bad word checking.
    if not default_moderator.allow(comment, content_object, request):
        # Comment will be disallowed outright (HTTP 403 response)
        return False

    if default_moderator.moderate(comment, content_object, request):
        comment.is_public = False


@receiver(signals.comment_was_posted)
def on_comment_posted(sender, comment, request, **kwargs):
    """
    Send email notification of a new comment to site staff when email notifications have been requested.
    """
    content_object = comment.content_object

    moderator = moderation.get_model_moderator(content_object.__class__)
    if moderator is None or comment.__class__ is not CommentModel:
        # No custom moderator means no email would be sent.
        # This still pass the comment to the default moderator.
        default_moderator.email(comment, content_object, request)



from django.conf.urls import url, include
import django_comments.urls

from . import views

urlpatterns = [
    url(r'^post/ajax/$', views.post_comment_ajax, name='comments-post-comment-ajax'),
    url(r'', include(django_comments.urls)),
]



import json
import sys

import django_comments
from django.apps import apps
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.http import HttpResponse, HttpResponseBadRequest
from django.template.loader import render_to_string
from django.utils.html import escape
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.http import require_POST
from django_comments import signals
from django_comments.views.comments import CommentPostBadRequest

from fluent_comments.compat import is_authenticated
from fluent_comments.utils import get_comment_template_name, get_comment_context_data
from fluent_comments import appsettings

if sys.version_info[0] >= 3:
    long = int


@csrf_protect
@require_POST
def post_comment_ajax(request, using=None):
    """
    Post a comment, via an Ajax call.
    """
    if not request.is_ajax():
        return HttpResponseBadRequest("Expecting Ajax call")

    # This is copied from django_comments.
    # Basically that view does too much, and doesn't offer a hook to change the rendering.
    # The request object is not passed to next_redirect for example.
    #
    # This is a separate view to integrate both features. Previously this used django-ajaxcomments
    # which is unfortunately not thread-safe (it it changes the comment view per request).

    # Fill out some initial data fields from an authenticated user, if present
    data = request.POST.copy()
    if is_authenticated(request.user):
        if not data.get('name', ''):
            data["name"] = request.user.get_full_name() or request.user.username
        if not data.get('email', ''):
            data["email"] = request.user.email

    # Look up the object we're trying to comment about
    ctype = data.get("content_type")
    object_pk = data.get("object_pk")
    if ctype is None or object_pk is None:
        return CommentPostBadRequest("Missing content_type or object_pk field.")
    try:
        model = apps.get_model(*ctype.split(".", 1))
        target = model._default_manager.using(using).get(pk=object_pk)
    except ValueError:
        return CommentPostBadRequest("Invalid object_pk value: {0}".format(escape(object_pk)))
    except (TypeError, LookupError):
        return CommentPostBadRequest("Invalid content_type value: {0}".format(escape(ctype)))
    except AttributeError:
        return CommentPostBadRequest("The given content-type {0} does not resolve to a valid model.".format(escape(ctype)))
    except ObjectDoesNotExist:
        return CommentPostBadRequest("No object matching content-type {0} and object PK {1} exists.".format(escape(ctype), escape(object_pk)))
    except (ValueError, ValidationError) as e:
        return CommentPostBadRequest("Attempting go get content-type {0!r} and object PK {1!r} exists raised {2}".format(escape(ctype), escape(object_pk), e.__class__.__name__))

    # Do we want to preview the comment?
    is_preview = "preview" in data

    # Construct the comment form
    form = django_comments.get_form()(target, data=data, is_preview=is_preview)

    # Check security information
    if form.security_errors():
        return CommentPostBadRequest("The comment form failed security verification: {0}".format(form.security_errors()))

    # If there are errors or if we requested a preview show the comment
    if is_preview:
        comment = form.get_comment_object() if not form.errors else None
        return _ajax_result(request, form, "preview", comment, object_id=object_pk)
    if form.errors:
        return _ajax_result(request, form, "post", object_id=object_pk)

    # Otherwise create the comment
    comment = form.get_comment_object()
    comment.ip_address = request.META.get("REMOTE_ADDR", None)
    if is_authenticated(request.user):
        comment.user = request.user

    # Signal that the comment is about to be saved
    responses = signals.comment_will_be_posted.send(
        sender=comment.__class__,
        comment=comment,
        request=request
    )

    for (receiver, response) in responses:
        if response is False:
            return CommentPostBadRequest("comment_will_be_posted receiver {0} killed the comment".format(receiver.__name__))

    # Save the comment and signal that it was saved
    comment.save()
    signals.comment_was_posted.send(
        sender  = comment.__class__,
        comment = comment,
        request = request
    )

    return _ajax_result(request, form, "post", comment, object_id=object_pk)


def _ajax_result(request, form, action, comment=None, object_id=None):
    # Based on django-ajaxcomments, BSD licensed.
    # Copyright (c) 2009 Brandon Konkle and individual contributors.
    #
    # This code was extracted out of django-ajaxcomments because
    # django-ajaxcomments is not threadsafe, and it was refactored afterwards.

    success = True
    json_errors = {}

    if form.errors:
        for field_name in form.errors:
            field = form[field_name]
            json_errors[field_name] = _render_errors(field)
        success = False

    json_return = {
        'success': success,
        'action': action,
        'errors': json_errors,
        'object_id': object_id,
        'use_threadedcomments': bool(appsettings.USE_THREADEDCOMMENTS),
    }

    if comment is not None:
        # Render the comment, like {% render_comment comment %} does
        context = get_comment_context_data(comment, action)
        context['request'] = request
        template_name = get_comment_template_name(comment)

        comment_html = render_to_string(template_name, context, request=request)
        json_return.update({
            'html': comment_html,
            'comment_id': comment.id,
            'parent_id': None,
            'is_moderated': not comment.is_public,   # is_public flags changes in comment_will_be_posted
        })
        if appsettings.USE_THREADEDCOMMENTS:
            json_return['parent_id'] = comment.parent_id

    json_response = json.dumps(json_return)
    return HttpResponse(json_response, content_type="application/json")


def _render_errors(field):
    """
    Render form errors in crispy-forms style.
    """
    template = '{0}/layout/field_errors.html'.format(appsettings.CRISPY_TEMPLATE_PACK)
    return render_to_string(template, {
        'field': field,
        'form_show_errors': True,
    })



from __future__ import absolute_import

import logging

from akismet import SpamStatus
from django_comments.moderation import moderator, CommentModerator

from fluent_comments import appsettings
from fluent_comments.akismet import akismet_check
from fluent_comments.email import send_comment_posted
from fluent_comments.utils import split_words

try:
    from urllib.parse import urljoin  # Python 3
except ImportError:
    from urlparse import urljoin  # Python 2

logger = logging.getLogger(__name__)

# Akismet code originally based on django-comments-spamfighter.

__all__ = (
    'FluentCommentsModerator',
    'moderate_model',
    'get_model_moderator',
    'comments_are_open',
    'comments_are_moderated',
)


class FluentCommentsModerator(CommentModerator):
    """
    Moderation policy for fluent-comments.
    """
    auto_close_field = None
    auto_moderate_field = None
    enable_field = None

    close_after = appsettings.FLUENT_COMMENTS_CLOSE_AFTER_DAYS
    moderate_after = appsettings.FLUENT_COMMENTS_MODERATE_AFTER_DAYS
    email_notification = appsettings.FLUENT_COMMENTS_USE_EMAIL_NOTIFICATION
    akismet_check = appsettings.FLUENT_CONTENTS_USE_AKISMET
    akismet_check_action = appsettings.FLUENT_COMMENTS_AKISMET_ACTION
    moderate_bad_words = set(appsettings.FLUENT_COMMENTS_MODERATE_BAD_WORDS)

    def allow(self, comment, content_object, request):
        """
        Determine whether a given comment is allowed to be posted on a given object.

        Returns ``True`` if the comment should be allowed, ``False`` otherwise.
        """
        # Parent class check
        if not super(FluentCommentsModerator, self).allow(comment, content_object, request):
            return False

        # Akismet check
        if self.akismet_check:
            akismet_result = akismet_check(comment, content_object, request)
            if self.akismet_check_action == 'delete' and akismet_result in (SpamStatus.ProbableSpam, SpamStatus.DefiniteSpam):
                return False  # Akismet marked the comment as spam.
            elif self.akismet_check_action == 'auto' and akismet_result == SpamStatus.DefiniteSpam:
                return False  # Clearly spam

        return True

    def moderate(self, comment, content_object, request):
        """
        Determine whether a given comment on a given object should be allowed to show up immediately,
        or should be marked non-public and await approval.

        Returns ``True`` if the comment should be moderated (marked non-public), ``False`` otherwise.
        """

        # Soft delete checks are done first, so these comments are not mistakenly "just moderated"
        # for expiring the `close_after` date, but correctly get marked as spam instead.
        # This helps staff to quickly see which comments need real moderation.
        if self.akismet_check:
            akismet_result = akismet_check(comment, content_object, request)
            if akismet_result:
                # Typically action=delete never gets here, unless the service was having problems.
                if akismet_result in (SpamStatus.ProbableSpam, SpamStatus.DefiniteSpam) and \
                       self.akismet_check_action in ('auto', 'soft_delete', 'delete'):
                   comment.is_removed = True  # Set extra marker

                # SpamStatus.Unknown or action=moderate will end up in the moderation queue
                return True

        # Parent class check
        if super(FluentCommentsModerator, self).moderate(comment, content_object, request):
            return True

        # Bad words check
        if self.moderate_bad_words:
            input_words = split_words(comment.comment)
            if self.moderate_bad_words.intersection(input_words):
                return True

        # Akismet check
        if self.akismet_check and self.akismet_check_action not in ('soft_delete', 'delete'):
            # Return True if akismet marks this comment as spam and we want to moderate it.
            if akismet_check(comment, content_object, request):
                return True

        return False

    def email(self, comment, content_object, request):
        """
        Overwritten for a better email notification.
        """
        if not self.email_notification:
            return

        send_comment_posted(comment, request)


class NullModerator(FluentCommentsModerator):
    """
    A moderator class that has the same effect as not being here.
    It allows all comments, disabling all moderation.
    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
    """

    def allow(self, comment, content_object, request):
        return True

    def moderate(self, comment, content_object, request):
        logger.info("Unconditionally allow comment, no default moderation set.")
        return False


class AlwaysModerate(FluentCommentsModerator):
    """
    A moderator class that will always mark the comment as moderated.
    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
    """
    def moderate(self, comment, content_object, request):
        # Still calling super in case Akismet marks the comment as spam.
        return super(AlwaysModerate, self).moderate(comment, content_object, request) or True


class AlwaysDeny(FluentCommentsModerator):
    """
    A moderator that will deny any comments to be posted.
    This can be used in ``FLUENT_COMMENTS_DEFAULT_MODERATOR``.
    """
    def allow(self, comment, content_object, request):
        logger.warning(
            "Discarded comment on unregistered model '%s'",
            content_object.__class__.__name__
        )
        return False


def moderate_model(ParentModel, publication_date_field=None, enable_comments_field=None):
    """
    Register a parent model (e.g. ``Blog`` or ``Article``) that should receive comment moderation.

    :param ParentModel: The parent model, e.g. a ``Blog`` or ``Article`` model.
    :param publication_date_field: The field name of a :class:`~django.db.models.DateTimeField` in the parent model which stores the publication date.
    :type publication_date_field: str
    :param enable_comments_field: The field name of a :class:`~django.db.models.BooleanField` in the parent model which stores the whether comments are enabled.
    :type enable_comments_field: str
    """
    attrs = {
        'auto_close_field': publication_date_field,
        'auto_moderate_field': publication_date_field,
        'enable_field': enable_comments_field,
    }
    ModerationClass = type(ParentModel.__name__ + 'Moderator', (FluentCommentsModerator,), attrs)
    moderator.register(ParentModel, ModerationClass)


def get_model_moderator(model):
    """
    Return the moderator class that is registered with a content object.
    If there is no associated moderator with a class, None is returned.

    :param model: The Django model registered with :func:`moderate_model`
    :type model: :class:`~django.db.models.Model`
    :return: The moderator class which holds the moderation policies.
    :rtype: :class:`~django_comments.moderation.CommentModerator`
    """
    try:
        return moderator._registry[model]
    except KeyError:
        return None


def comments_are_open(content_object):
    """
    Return whether comments are still open for a given target object.
    """
    moderator = get_model_moderator(content_object.__class__)
    if moderator is None:
        return True

    # Check the 'enable_field', 'auto_close_field' and 'close_after',
    # by reusing the basic Django policies.
    return CommentModerator.allow(moderator, None, content_object, None)


def comments_are_moderated(content_object):
    """
    Return whether comments are moderated for a given target object.
    """
    moderator = get_model_moderator(content_object.__class__)
    if moderator is None:
        return False

    # Check the 'auto_moderate_field', 'moderate_after',
    # by reusing the basic Django policies.
    return CommentModerator.moderate(moderator, None, content_object, None)



"""
Variations of the form that use
"""
from __future__ import absolute_import

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from fluent_comments.forms._captcha import CaptchaFormMixin
from .compact import CompactCommentForm, CompactLabelsCommentForm
from .default import DefaultCommentForm

try:
    from nocaptcha_recaptcha.fields import NoReCaptchaField

    captcha_field = NoReCaptchaField()
except ImportError:
    try:
        from captcha.fields import ReCaptchaField
        captcha_field = ReCaptchaField()

        if not getattr(settings, 'NOCAPTCHA', False):
            raise ImproperlyConfigured(
                'reCAPTCHA v1 is phased out. Add `NOCAPTCHA = True` to your settings '
                'to use the modern "no captcha" reCAPTCHA v2.'
            )
    except ImportError:
        raise ImportError(
            "To use '{}', you need to have django-nocaptcha-recaptcha"
            " or django-recaptcha2 installed.".format(__name__)
        )


class DefaultCommentForm(CaptchaFormMixin, DefaultCommentForm):
    """
    Contact form with reCAPTCHA field.
    """
    captcha = captcha_field

    class Media:
        js = (
            'https://www.google.com/recaptcha/api.js',
        )


class CompactCommentForm(CaptchaFormMixin, CompactCommentForm):
    """
    Compact variation 1.
    """
    captcha = captcha_field

    class Media:
        js = (
            'https://www.google.com/recaptcha/api.js',
        )


class CompactLabelsCommentForm(CaptchaFormMixin, CompactLabelsCommentForm):
    """
    Compact variation 2.
    """
    captcha = captcha_field

    class Media:
        js = (
            'https://www.google.com/recaptcha/api.js',
        )



from __future__ import absolute_import

from akismet import Akismet, SpamStatus
from django.core.exceptions import ImproperlyConfigured
from django.utils.encoding import smart_str

import fluent_comments
from fluent_comments import appsettings

try:
    from urllib.parse import urljoin  # Python 3
except ImportError:
    from urlparse import urljoin  # Python 2

try:
    from django.contrib.sites.shortcuts import get_current_site  # Django 1.9+
except ImportError:
    from django.contrib.sites.models import get_current_site


def akismet_check(comment, content_object, request):
    """
    Connects to Akismet and evaluates to True if Akismet marks this comment as spam.

    :rtype: akismet.SpamStatus
    """
    # Return previously cached response
    akismet_result = getattr(comment, '_akismet_result_', None)
    if akismet_result is not None:
        return akismet_result

    # Get Akismet data
    AKISMET_API_KEY = appsettings.AKISMET_API_KEY
    if not AKISMET_API_KEY:
        raise ImproperlyConfigured('You must set AKISMET_API_KEY to use comment moderation with Akismet.')

    current_domain = get_current_site(request).domain
    auto_blog_url = '{0}://{1}/'.format(request.is_secure() and 'https' or 'http', current_domain)
    blog_url = appsettings.AKISMET_BLOG_URL or auto_blog_url

    akismet = Akismet(
        AKISMET_API_KEY,
        blog=blog_url,
        is_test=int(bool(appsettings.AKISMET_IS_TEST)),
        application_user_agent='django-fluent-comments/{0}'.format(fluent_comments.__version__),
    )

    akismet_data = _get_akismet_data(blog_url, comment, content_object, request)
    akismet_result = akismet.check(**akismet_data)  # raises AkismetServerError when key is invalid
    setattr(comment, "_akismet_result_", akismet_result)
    return akismet_result


def _get_akismet_data(blog_url, comment, content_object, request):
    # Field documentation:
    # http://akismet.com/development/api/#comment-check
    data = {
        # Comment info
        'permalink': urljoin(blog_url, content_object.get_absolute_url()),
        # see http://blog.akismet.com/2012/06/19/pro-tip-tell-us-your-comment_type/
        'comment_type': 'comment',  # comment, trackback, pingback
        'comment_author': getattr(comment, 'name', ''),
        'comment_author_email': getattr(comment, 'email', ''),
        'comment_author_url': getattr(comment, 'url', ''),
        'comment_content': smart_str(comment.comment),
        'comment_date': comment.submit_date,

        # Request info
        'referrer': request.META.get('HTTP_REFERER', ''),
        'user_agent': request.META.get('HTTP_USER_AGENT', ''),
        'user_ip': comment.ip_address,
    }

    if comment.user_id and comment.user.is_superuser:
        data['user_role'] = 'administrator'  # always passes test

    # If the language is known, provide it.
    language = _get_article_language(content_object)
    if language:
        data['blog_lang'] = language

    return data


def _get_article_language(article):
    try:
        # django-parler uses this attribute
        return article.get_current_language()
    except AttributeError:
        pass

    try:
        return article.language_code
    except AttributeError:
        pass

    return None



"""
Variations of the form that use
"""
from __future__ import absolute_import

from django.utils.translation import pgettext_lazy

from fluent_comments.forms._captcha import CaptchaFormMixin
from .compact import CompactCommentForm, CompactLabelsCommentForm
from .default import DefaultCommentForm

try:
    from captcha.fields import CaptchaField
except ImportError:
    raise ImportError(
        "To use '{}', you need to have django-simple-captcha installed.".format(__name__)
    )

captcha_field = CaptchaField(help_text=pgettext_lazy("captcha-help-text", u"Type the text."))


class DefaultCommentForm(CaptchaFormMixin, DefaultCommentForm):
    """
    Comment form with reCAPTCHA field.
    """
    captcha = captcha_field


class CompactCommentForm(CaptchaFormMixin, CompactCommentForm):
    """
    Compact variation 1.
    """
    captcha = captcha_field


class CompactLabelsCommentForm(CaptchaFormMixin, CompactLabelsCommentForm):
    """
    Compact variation 2.
    """
    captcha = captcha_field



from .base import AbstractCommentForm, CommentFormHelper
from .default import DefaultCommentForm
from .compact import CompactLabelsCommentForm, CompactCommentForm
from .helper import CommentFormHelper, SubmitButton, PreviewButton, CompactLabelsCommentFormHelper

FluentCommentForm = DefaultCommentForm  # noqa, for backwards compatibility

__all__ = (
    'AbstractCommentForm',
    'CommentFormHelper',
    'DefaultCommentForm',
    'CompactLabelsCommentFormHelper',
    'CompactLabelsCommentForm',
    'CompactCommentForm',
    'SubmitButton',
    'PreviewButton',
)



"""Compatibility fixes that can't be resolved with an simple ``except ImportError``"""
import django


if django.VERSION >= (1, 10):
    def is_authenticated(user):
        return user.is_authenticated  # CallableBool or bool
else:
    def is_authenticated(user):
        return user.is_authenticated()  # Method




from django.core.exceptions import ImproperlyConfigured


class CaptchaFormMixin(object):
    def _reorder_fields(self, ordering):
        """
        Test that the 'captcha' field is really present.
        This could be broken by a bad FLUENT_COMMENTS_FIELD_ORDER configuration.
        """
        if 'captcha' not in ordering:
            raise ImproperlyConfigured(
                "When using 'FLUENT_COMMENTS_FIELD_ORDER', "
                "make sure the 'captcha' field included too to use '{}' form. ".format(
                    self.__class__.__name__
                )
            )
        super(CaptchaFormMixin, self)._reorder_fields(ordering)

        # Avoid making captcha required for previews.
        if self.is_preview:
            self.fields.pop('captcha')



from fluent_comments.forms.base import AbstractCommentForm, CommentFormHelper


class DefaultCommentForm(AbstractCommentForm):
    """
    A simple comment form, backed by a model to save all data (in case email fails).
    """
    helper = CommentFormHelper()



"""
Different form layout, very compact
"""
from crispy_forms.layout import Column, Layout, Row
from django.utils.functional import cached_property

from fluent_comments import appsettings
from fluent_comments.forms.base import AbstractCommentForm, PreviewButton, SubmitButton
from fluent_comments.forms.helper import CompactLabelsCommentFormHelper


class CompactLabelsCommentForm(AbstractCommentForm):
    """
    A form layout where the labels are replaced with ``placeholder`` attributes.
    """

    @cached_property
    def helper(self):
        # Initialize on demand
        helper = CompactLabelsCommentFormHelper()
        helper.layout = Layout(*self.fields.keys())
        helper.add_input(SubmitButton())
        helper.add_input(PreviewButton())
        return helper


class CompactCommentForm(AbstractCommentForm):
    """
    A form with a very compact layout;
    all the name/email/phone fields are displayed in a single top row.
    It uses Bootstrap 3 layout by default to generate the columns.
    """
    top_row_fields = appsettings.FLUENT_COMMENTS_COMPACT_FIELDS
    top_row_columns = appsettings.FLUENT_COMMENTS_COMPACT_GRID_SIZE
    top_column_class = appsettings.FLUENT_COMMENTS_COMPACT_COLUMN_CSS_CLASS

    @cached_property
    def helper(self):
        # As extra service, auto-adjust the layout based on the project settings.
        # This allows defining the top-row, and still get either 2 or 3 columns
        compact_fields = [name for name in self.fields.keys() if name in self.top_row_fields]
        other_fields = [name for name in self.fields.keys() if name not in self.top_row_fields]
        col_size = int(self.top_row_columns / len(compact_fields))
        col_class = self.top_column_class.format(size=col_size)

        compact_row = Row(*[Column(name, css_class=col_class) for name in compact_fields])

        # The fields are already ordered by the AbstractCommentForm.__init__ method.
        # See where the compact row should be.
        pos = list(self.fields.keys()).index(compact_fields[0])
        new_fields = other_fields
        new_fields.insert(pos, compact_row)

        helper = CompactLabelsCommentFormHelper()
        helper.layout = Layout(*new_fields)
        helper.add_input(SubmitButton())
        helper.add_input(PreviewButton())
        return helper



"""
Internal utils
"""
import re

from django.contrib.contenttypes.models import ContentType

from fluent_comments import appsettings


RE_INTERPUNCTION = re.compile(r'\W+')


def get_comment_template_name(comment):
    """
    Internal function for the rendering of comments.
    """
    ctype = ContentType.objects.get_for_id(comment.content_type_id)
    return [
        "comments/%s/%s/comment.html" % (ctype.app_label, ctype.model),
        "comments/%s/comment.html" % ctype.app_label,
        "comments/comment.html"
    ]


def get_comment_context_data(comment, action=None):
    """
    Internal function for the rendering of comments.
    """
    return {
        'comment': comment,
        'action': action,
        'preview': (action == 'preview'),
        'USE_THREADEDCOMMENTS': appsettings.USE_THREADEDCOMMENTS,
    }


def split_words(comment):
    """
    Internal function to split words
    """
    return set(RE_INTERPUNCTION.sub(' ', comment).split())



from django.utils.translation import ugettext_lazy as _
from django_comments import get_model as get_comments_model
from django_comments.managers import CommentManager

from fluent_comments import appsettings

try:
    from django.contrib.contenttypes.fields import GenericRelation  # Django 1.9+
except ImportError:
    from django.contrib.contenttypes.generic import GenericRelation

if appsettings.USE_THREADEDCOMMENTS:
    from threadedcomments.models import ThreadedComment as BaseModel
else:
    from django_comments.models import Comment as BaseModel


class FluentCommentManager(CommentManager):
    """
    Manager to optimize SQL queries for comments.
    """

    def get_queryset(self):
        return super(CommentManager, self).get_queryset().select_related('user')


class FluentComment(BaseModel):
    """
    Proxy model to make sure that a ``select_related()`` is performed on the ``user`` field.
    """
    objects = FluentCommentManager()

    class Meta:
        verbose_name = _("Comment")
        verbose_name_plural = _("Comments")
        proxy = True
        managed = False


def get_comments_for_model(content_object, include_moderated=False):
    """
    Return the QuerySet with all comments for a given model.
    """
    qs = get_comments_model().objects.for_model(content_object)

    if not include_moderated:
        qs = qs.filter(is_public=True, is_removed=False)

    return qs


class CommentsRelation(GenericRelation):
    """
    A :class:`~django.contrib.contenttypes.generic.GenericRelation` which can be applied to a parent model that
    is expected to have comments. For example:

    .. code-block:: python

        class Article(models.Model):
            comments_set = CommentsRelation()
    """

    def __init__(self, *args, **kwargs):
        super(CommentsRelation, self).__init__(
            to=get_comments_model(),
            content_type_field='content_type',
            object_id_field='object_pk',
            **kwargs
        )



#!/usr/bin/env python
from __future__ import print_function
from setuptools import setup, find_packages
from os import path
import codecs
import os
import re
import sys


# When creating the sdist, make sure the django.mo file also exists:
if 'sdist' in sys.argv or 'develop' in sys.argv:
    os.chdir('fluent_comments')
    try:
        from django.core import management
        management.call_command('compilemessages', stdout=sys.stderr, verbosity=1)
    except ImportError:
        if 'sdist' in sys.argv:
            raise
    finally:
        os.chdir('..')


def read(*parts):
    file_path = path.join(path.dirname(__file__), *parts)
    return codecs.open(file_path, encoding='utf-8').read()


def find_version(*parts):
    version_file = read(*parts)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
    if version_match:
        return str(version_match.group(1))
    raise RuntimeError("Unable to find version string.")


setup(
    name='django-fluent-comments',
    version=find_version('fluent_comments', '__init__.py'),
    license='Apache 2.0',

    install_requires = [
        'django-crispy-forms>=1.1.1',
        'django-tag-parser>=3.1',
        'django-contrib-comments>=1.8',
        'python-akismet>=0.4.1',  # Python 3 port, replaces Python 2-only "akismet" library.
    ],
    requires=[
        'Django (>=1.8)',
    ],
    tests_require = [
        'mock',
    ],
    extras_require = {
        'threadedcomments': ['django-threadedcomments>=1.1'],
    },
    description='A modern, ajax-based appearance for django_comments',
    long_description=read('README.rst'),

    author='Diederik van der Boor',
    author_email='opensource@edoburu.nl',

    url='https://github.com/edoburu/django-fluent-comments',
    download_url='https://github.com/edoburu/django-fluent-comments/zipball/master',

    packages=find_packages(exclude=('example*',)),
    include_package_data=True,

    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



from django.test import SimpleTestCase

from fluent_comments.utils import split_words


class TestUtils(SimpleTestCase):
    def test_split_words(self):
        text = """college scholarship essays - <a href=" https://collegeessays.us/ ">how to write a good introduction for a college essay</a> 
boston university college essay <a href=" https://collegeessays.us/ ">how to write an essay for college</a> 
https://collegeessays.us/ 
http://www.monkeyface.com/__media__/js/netsoltrademark.php?d=collegeessays.us"""
        self.assertEqual(split_words(text), {
            '__media__', 'a', 'an', 'boston', 'college', 'collegeessays',
            'com', 'd', 'essay', 'essays', 'for', 'good', 'how', 'href',
            'http', 'https', 'introduction', 'js', 'monkeyface', 'netsoltrademark',
            'php', 'scholarship', 'to', 'university', 'us', 'write', 'www'
        })



from django.conf import settings
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.encoding import force_text

try:
    from django.contrib.sites.shortcuts import get_current_site  # Django 1.9+
except ImportError:
    from django.contrib.sites.models import get_current_site


def send_comment_posted(comment, request):
    """
    Send the email to staff that an comment was posted.

    While the django_comments module has email support,
    it doesn't pass the 'request' to the context.
    This also changes the subject to show the page title.
    """
    recipient_list = [manager_tuple[1] for manager_tuple in settings.MANAGERS]
    site = get_current_site(request)
    content_object = comment.content_object
    content_title = force_text(content_object)

    if comment.is_removed:
        subject = u'[{0}] Spam comment on "{1}"'.format(site.name, content_title)
    elif not comment.is_public:
        subject = u'[{0}] Moderated comment on "{1}"'.format(site.name, content_title)
    else:
        subject = u'[{0}] New comment posted on "{1}"'.format(site.name, content_title)

    context = {
        'site': site,
        'comment': comment,
        'content_object': content_object
    }

    message = render_to_string("comments/comment_notification_email.txt", context, request=request)
    send_mail(subject, message, settings.DEFAULT_FROM_EMAIL, recipient_list, fail_silently=True)



from functools import wraps

import fluent_comments
from fluent_comments import appsettings
from fluent_comments.moderation import FluentCommentsModerator


def override_appsettings(**settings):
    """
    Temporary override the appsettings.
    """
    def _dec(func):
        @wraps(func)
        def _inner(*args, **kwargs):
            # Apply new settings, backup old, clear caches
            old_values = {}
            for key, new_value in settings.items():
                old_values[key] = getattr(appsettings, key)
                setattr(appsettings, key, new_value)
            _reset_setting_caches()

            func(*args, **kwargs)
            for key, old_value in old_values.items():
                setattr(appsettings, key, old_value)

            # reset caches
            _reset_setting_caches()
        return _inner
    return _dec


def _reset_setting_caches():
    fluent_comments.form_class = None
    fluent_comments.model_class = None
    FluentCommentsModerator.close_after = appsettings.FLUENT_COMMENTS_CLOSE_AFTER_DAYS
    FluentCommentsModerator.moderate_after = appsettings.FLUENT_COMMENTS_MODERATE_AFTER_DAYS
    FluentCommentsModerator.akismet_check = appsettings.FLUENT_CONTENTS_USE_AKISMET
    FluentCommentsModerator.akismet_check_action = appsettings.FLUENT_COMMENTS_AKISMET_ACTION
    FluentCommentsModerator.moderate_bad_words = set(appsettings.FLUENT_COMMENTS_MODERATE_BAD_WORDS)


class MockedResponse(object):
    def __init__(self, result, definitive=False):
        self.result = result
        self.headers = {}
        if definitive:
            self.headers['X-Akismet-Pro-Tip'] = 'discard'

    def json(self):
        return self.result
